import bb.cascades 1.2
import bb.system 1.2

Sheet {
    onCreationCompleted: {
        app.categoriesLoaded.connect(populateCategories);
        app.refreshCategories();
    }
    id: root
    property alias title : title_text.text
    property alias title_bar : titlebar.title
    property alias category : category_dd
    property string category_title
    property int note_id: 0
    property string shareString : ""
    
    function setContent(content){
        content_text.setText(content);
    }
    function getContent(){
        return content_text.text;
    }
    function setShareString(){
        var mcategory = new String("");
        if(title_text.text && content_text.text)
        {
            if(category.selectedIndex == -1)
                mcategory = "";
            else
                mcategory = category.selectedOption.text;
            
            shareString = qsTr("Title:") + Retranslate.onLocaleOrLanguageChanged + " " +  title_text.text + "\n" + qsTr("Note:") + Retranslate.onLocaleOrLanguageChanged  + " " + content_text.text;
            if (mcategory) {
                shareString = shareString + "\n[" + mcategory + "]";
                shareString = shareString + "\n" + qsTr("Shared using Secure Notes on BB10") + Retranslate.onLocaleOrLanguageChanged
            }                        
        }
        else 
            shareString = qsTr("Nothing to share (yet)") + Retranslate.onLocaleOrLanguageChanged
        console.debug("Setting String: " + shareString);
    }
    function buildString()
    {
        var newString = new String("");
        newString = title_text.text +":"+ content_text.text;
        console.debug(newString);
        // qShare.query.data = newString;
        shareString = newString;
        console.debug("Created string: " + newString);
    }
    function openNote(Note){
        title = Note.Title;
        title_bar = Note.Title;
        setContent(Note.Content);
        category_title = Note.Category;
        note_id = Note.Id;
        for(var i = 0; i < category.options.length; ++i){
            if(category.options[i].text == category_title){
                category.setSelectedIndex(i);
            }
        }       
        open();
    }
    function populateCategories(categories_list){
        for(var i in categories_list){
            var added = new_option.createObject(category);
            added.text = categories_list[i].Title;
            category.add(added);
        }
        app.categoriesLoaded.disconnect(populateCategories);
    }
    onClosed: {
        destroy();
    }
    Page {
        titleBar: TitleBar {
            title: qsTr("Creating New Note") + Retranslate.onLocaleOrLanguageChanged
            id: titlebar
            acceptAction: ActionItem {
                title: qsTr("Save") + Retranslate.onLocaleOrLanguageChanged
                onTriggered: {
                    console.log("Title: " + title);
                    if(root.title.length > 0){
                        var cat_text;
                        if(category.selectedIndex == -1){
                            cat_text = "";
                        }
                        else{
                            cat_text = category.selectedOption.text;
                        }
                        var note = new Object();
                        note.Title = title_text.text;
                        note.Content = content_text.text;
                        note.Category = cat_text;
                        note.Id = note_id;
                        note.Tags = "";
                        app.saveNote(note);
                        parent.parent.parent.close();
                    }
                    else{
                        toast.body = qsTr("Notes cannot be saved with an empty title.") + Retranslate.onLocaleOrLanguageChanged;
                        toast.show();
                    }
                }
            }
            dismissAction: ActionItem {
                title: "Close"
                onTriggered: {
                    parent.parent.parent.close();
                }
            }
        }
        actions: [
            ActionItem {
                title: qsTr("Save") + Retranslate.onLocaleOrLanguageChanged
                imageSource: "asset:///icons/a_export.png"
                ActionBar.placement: ActionBarPlacement.Signature
                shortcuts: [ Shortcut { key: "S" }, Shortcut { key: "s" }]            
                onTriggered: {
                    console.log("Title: " + titlebar.title);
                    if(root.title.length > 0){
                        var cat_text;
                        if(category.selectedIndex == -1){
                            cat_text = "";
                        }
                        else{
                            cat_text = category.selectedOption.text;
                        }
                        var note = new Object();
                        note.Title = title_text.text;
                        note.Content = content_text.text;
                        note.Category = cat_text;
                        note.Id = note_id;
                        note.Tags = "";
                        app.saveNote(note);
                        
                        toast.body = qsTr("Changes Saved") + Retranslate.onLocaleOrLanguageChanged;
                        toast.show();
                        
                    }
                    else{
                        toast.body = qsTr("Notes cannot be saved with an empty title.") + Retranslate.onLocaleOrLanguageChanged;
                        toast.show();
                    }
                }   
            },
            ActionItem {
                title: qsTr("Delete") + Retranslate.onLocaleOrLanguageChanged
                enabled: note_id != 0
                imageSource: "asset:///icons/a_remove.png"
                ActionBar.placement: ActionBarPlacement.InOverflow
                shortcuts: [ Shortcut { key: "D" }, Shortcut { key: "d" }]
                onTriggered: {
                    deleteConfirmDialog.show();
                }
                                
                attachedObjects: [                
                    SystemDialog {
                        id: deleteConfirmDialog
                        
                        title: qsTr("Deleting Entry") + Retranslate.onLocaleOrLanguageChanged
                        body: qsTr("Are you sure you want to delete this entry?  This cannot be undone.") + Retranslate.onLocaleOrLanguageChanged
                        customButton.enabled: false
                        confirmButton.label: qsTr("Delete") + Retranslate.onLocaleOrLanguageChanged
                        
                        onFinished: {
                            
                            if (deleteConfirmDialog.result == SystemUiResult.ConfirmButtonSelection) {
                                var note = new Object();
                                note.Id = note_id;
                                note.Title = root.title;
                                note.Content = "";
                                note.Tags = "";
                                note.Category = "";
                                app.deleteNote(note);
                                root.close();
                            }
                        }
                    }
                ]
                
            },
            ActionItem {
                title: qsTr("Share Note") + Retranslate.onLocaleOrLanguageChanged
                imageSource: "asset:///icons/a_share.png"
                ActionBar.placement: ActionBarPlacement.InOverflow
                shortcuts: [ Shortcut { key: "H" }, Shortcut { key: "h" }]            
                onTriggered: {
                    setShareString();
                    shareQuery.trigger("bb.action.SHARE")
                    query.mimeType= "text/plain"
                    query.data = shareString;
                    query.updateQuery();
                }   
            }
        ]
        attachedObjects: [
            UiConfig {
                id: to  
            },
            ComponentDefinition {
                id: new_option
                Option {
                
                }
            },
            SystemToast {
                id: toast
            },
            SystemPrompt {
                function contains(myString, pattern){
                    return !(myString.indexOf(pattern) == -1);    
                
                }
                function exists(title){
                    for(var i = 0; i < category.options.length; ++i){
                        console.log(category.options[i].text);
                        if(category.options[i].text == title){
                            return true;
                        }
                    }
                    return false;
                }
                id: categories_prompt
                title: qsTr("Enter Category Name: ") + Retranslate.onLocaleOrLanguageChanged
                onFinished: {
                    if(result == SystemUiResult.ConfirmButtonSelection){
                        if(!contains(inputFieldTextEntry(), "'") &&
                        !contains(inputFieldTextEntry(), "\"") &&
                        inputFieldTextEntry() != "Add new..." &&
                        !exists(inputFieldTextEntry())
                        ){
                            var added = new_option.createObject(category);
                            added.text = inputFieldTextEntry();
                            added.value = inputFieldTextEntry();
                            category.add(added);
                            category.setSelectedOption(added);
                            app.saveCategory(inputFieldTextEntry());
                        }
                        else{
                            toast.body = qsTr("Invalid Name!") + Retranslate.onLocaleOrLanguageChanged;
                            toast.update();
                            toast.show();
                            category.setSelectedIndex(-1);
                        }
                    }
                    else{
                        category.setSelectedIndex(-1);
                    }
                }
            },
            Invocation {
                id: shareQuery
                query: InvokeQuery {
                    id: query
                }
                onArmed:{                    
                    trigger("bb.action.SHARE");                
                }
            }
        ]    
        Container {
            topPadding: to.sdu(1.0)
            leftPadding: to.sdu(1.0)
            rightPadding: to.sdu(1.0)
            bottomPadding: to.sdu(1.0)
            TextField {
                id: title_text
                hintText: qsTr("Enter a Title") + Retranslate.onLocaleOrLanguageChanged
            }            
            DropDown {
                title: qsTr("Optional: Categorize Note") + Retranslate.onLocaleOrLanguageChanged
                objectName: "category_dd"
                id: category_dd
                selectedIndex: -1
                onSelectedValueChanged: {
                    if(selectedValue == -1){
                        categories_prompt.show();
                    }
                }
                options: [
                    Option {
                        text: qsTr("Add new category...") + Retranslate.onLocaleOrLanguageChanged
                        value: -1
                    }
                ]
            }
            TextArea {
                id: content_text
                hintText: qsTr("Enter your Note") + Retranslate.onLocaleOrLanguageChanged
                horizontalAlignment: HorizontalAlignment.Fill
                verticalAlignment: VerticalAlignment.Fill
                minHeight: to.sdu(40.0)
            }
        
        }
    }
}