import bb.cascades 1.2
import bb.system 1.0
import bb.cascades.pickers 1.0

Page {
    function exitApp(){
        Application.setCover(0);
        Application.quit();
    }
    onCreationCompleted: {
        Application.thumbnail.connect(active_frame.setText);
        Application.setCover(active_frame);
        Application.manualExit.connect(exitApp);
        Application.setAutoExit(false);
    }
    
    id: root
    attachedObjects: [
        FilePicker {
            
            id: filePicker
            title : qsTr("Select File") + Retranslate.onLocaleOrLanguageChanged
            directories : app.getPath()
            viewMode: FilePickerViewMode.ListView
            allowOverwrite: true
            onPickerOpened: {
                filePicker.allowOverwrite = true;
            }
            onError: {
                toast.body = qsTr("An error occurred accessing the file") + Retranslate.onLocaleOrLanguageChanged;
                toast.show();
            }
            onFileSelected : {
                console.log("FileSelected signal received : " + selectedFiles);
                filePicker.allowOverwrite = true;
                
                if (mode == FilePickerMode.Saver)
                {
                    
                    selectedFiles.forEach(function(entry) {
                            app.saveText(entry);
                    })
                }
                else if (mode == FilePickerMode.Picker)
                {
                    selectedFiles.forEach(function(entry) {
                            app.loadText(entry);
                    });
                }
                app.onInvokeRefresh();
            }
            objectName: "m_filePicker"
        },
        SystemToast {
            id: toast
        },
        ComponentDefinition {
            source: "asset:///AboutPage.qml"
            id: about_page
        },
        ComponentDefinition {
            source: "asset:///NoteEditor.qml"
            id: note_editor
        },
        ComponentDefinition {
            source: "asset:///SettingsPage.qml"
            id: settings_page
        },
        UiConfig {
            id: to
        },
        ActiveFrame{
            id: active_frame
        },
        Invocation {
            id: shareQuery
            query {
                mimeType: "text/plain"
                data: qsTr("Password protect your notes and keep them secure with Secure Notes http://appworld.blackberry.com/webstore/content/59948562 from FileArchiveHaven http://appworld.blackberry.com/webstore/vendor/18376") + Retranslate.onLocaleOrLanguageChanged
            }
        }  
    ]
    titleBar: TitleBar {
        property bool created: false
        kind: TitleBarKind.Segmented
        id: title_bar
        onCreationCompleted: {
            created = true;
        }
        onSelectedIndexChanged: {
            if(created){
                app.swapDatamodel();
            }
        }
        options: [
            Option {
                text: qsTr("Notes") + Retranslate.onLocaleOrLanguageChanged
            },
            Option {
                text: qsTr("Categories") + Retranslate.onLocaleOrLanguageChanged
            }
        ]
    }
    actions: [
        ActionItem {
            
            title: qsTr("New Note") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "asset:///icons/a_note.png"
            ActionBar.placement: ActionBarPlacement.Signature
            shortcuts: [ Shortcut { key: "N" }, Shortcut { key: "n" }]            
            onTriggered: {
                var noted = note_editor.createObject(root);
                noted.open();
            }
        },
        ActionItem {
            title: qsTr("Share App") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "asset:///icons/a_share.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            shortcuts: [ Shortcut { key: "S" }, Shortcut { key: "s" }]            
            onTriggered: {
                shareQuery.trigger("bb.action.SHARE")
            }   
        },
        ActionItem {
            title: qsTr("Report Bug") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "icons/a_bug.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            
            attachedObjects: [
                SystemDialog {
                    id: bugDialog
                    title: qsTr("Report Bug") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("Tip:  When submitting a bug report, try to include steps to reproduce the problem.") + Retranslate.onLocaleOrLanguageChanged
                    
                    onFinished: {
                        if (bugDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            bugQuery.trigger("bb.action.SENDEMAIL");
                        }
                    }
                },
                Invocation {
                    id: bugQuery
                    query {
                        mimeType: "text/plain"
                        invokeTargetId: "sys.pim.uib.email.hybridcomposer"
                        uri: "mailto:support@filearchivehaven.com?subject=Bug%20Report%20SecureNotes%20" + Application.applicationVersion
                    }                   
                }
            ]
            
            onTriggered: {
                bugDialog.show();
            }
        },
        ActionItem {
            title: qsTr("Known Bugs") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "icons/a_knownbug.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            shortcuts: [ Shortcut { key: "K" }, Shortcut { key: "K" }]
            
            onTriggered: {
                _downloadManager.getBugsOnline("http://www.filearchivehaven.com/data/SecureNotes/bugs")
            }
        },        
        ActionItem {
            title: qsTr("Write Review") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "icons/a_review.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            shortcuts: [ Shortcut { key: "W" }, Shortcut { key: "w" }]        
            
            attachedObjects: [
                SystemDialog {
                    id: reviewDialog
                    title: qsTr("Write Review") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("I need your help to keep making this application great!  If you experience any issues, please try contacting me before leaving a negative review (so I can fix the problem).  Please be honest in your review, but keep in mind that positive reviews means more users (and ultimately more improvements).") + Retranslate.onLocaleOrLanguageChanged 
                    onFinished: {
                        if (reviewDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            reviewQuery.trigger("bb.action.OPEN")
                        }
                    }
                },
                Invocation {
                    id: reviewQuery
                    query {
                        mimeType: "text/html"
                        invokeTargetId:  "sys.appworld"
                        uri: "appworld://content/59948562"
                    }                   
                }
            ]
            
            onTriggered: {
                reviewDialog.show()
            }
        },        
        ActionItem {
            title: qsTr("Import") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "icons/a_import.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            
            attachedObjects: [                
                SystemDialog {
                    id: wipedataDialog
                    
                    title: qsTr("Wiping Data") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("When importing a dataset, your current data will be wiped. To prevent data-loss, we strongly recommend exporting your current data as a backup prior to importing.") + Retranslate.onLocaleOrLanguageChanged
                    customButton.enabled: false
                    confirmButton.label: qsTr("OK") + Retranslate.onLocaleOrLanguageChanged
                    onFinished: {
                        if (wipedataDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            filePicker.mode = FilePickerMode.Picker;
                            filePicker.title = qsTr("Import") + Retranslate.onLocaleOrLanguageChanged                            
                            filePicker.open();
                        }
                    }
                }
            ]
            
            onTriggered: {
                wipedataDialog.show();
            }
        },        
        ActionItem {
            title: qsTr("Export") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "icons/a_export.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            
            attachedObjects: [                
                SystemDialog {
                    id: unencryptedDialog
                    
                    title: qsTr("Unencrypted Data") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("Please be aware, backups are not encrypted, and can be read by anyone.") + Retranslate.onLocaleOrLanguageChanged
                    customButton.enabled: false
                    cancelButton.enabled: false
                    cancelButton.label: undefined
                    confirmButton.label: qsTr("OK") + Retranslate.onLocaleOrLanguageChanged
                    
                    onFinished: {
                        
                        if (unencryptedDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            filePicker.mode = FilePickerMode.Saver;
                            filePicker.allowOverwrite = true;                            
                            filePicker.title = qsTr("Export") + Retranslate.onLocaleOrLanguageChanged
                            filePicker.defaultSaveFileNames = "secureNotes.bak";
                            filePicker.open();
                        }
                    }
                }
            ]
            
            onTriggered: {
                unencryptedDialog.show();
            }
        }
    ]
    Menu.definition: MenuDefinition {        
        settingsAction: SettingsActionItem {
            title: qsTr("Settings") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "asset:///icons/a_settings.png"
            shortcuts: [ Shortcut { key: "E" }, Shortcut { key: "e" }]            
            onTriggered: {
                var settings = settings_page.createObject(root);
                settings.open();
            }
        }    
        actions: [
            ActionItem {
                title: qsTr("About") + Retranslate.onLocaleOrLanguageChanged
                imageSource: "asset:///icons/a_about.png"
                shortcuts: [ Shortcut { key: "A" }, Shortcut { key: "a" }]            
                onTriggered: {
                    var about = about_page.createObject(root);
                    about.open();
                }
            }
        ]    
    }
    Container {
        
        Label {
            id: passcheck
            text: "Swipe down to access settings and set a password."
            visible: {
                var response = app.getValueFor("showMessage", true);
                if(app.getPasswordStatus()){
                    return true;
                }
                return response;
            }
            verticalAlignment: VerticalAlignment.Center
            textStyle.textAlign: TextAlign.Center
            onTouch: {
                console.log("pressed.");
                app.setValueFor("showMessage", false);
                passcheck.visible = app.getValueFor("showMessage", true);
            }
            
            textFormat: TextFormat.Plain
            horizontalAlignment: HorizontalAlignment.Center
            layoutProperties: FlowListLayoutProperties {
            
            }
            topMargin: 7.0
            rightMargin: 7.0
            leftMargin: 7.0
            bottomMargin: 7.0
        
        }
        
        ListView {
            function deleteNote(indexPath){
                app.deleteNote(dataModel.data(indexPath));
            }
            function deleteCategory(indexPath){
                app.deleteCategory(dataModel.data(indexPath)["Title"]);
            }
            function getSelectedIndex(){
                return title_bar.selectedIndex;
            }
            property alias to: to
            objectName: "main_list"
            id: note_list
            multiSelectAction: MultiSelectActionItem {
            
            }
            multiSelectHandler{
                actions: [
                    DeleteActionItem {
                        id: single_delete
                        onTriggered: {
                            if(!title_bar.selectedIndex){
                                app.deleteNotes(note_list.selectionList());
                            }
                            else{
                                app.deleteCategories(note_list.selectionList());
                            }
                        }
                    }
                ]
            }
            onTriggered: {
                if(!title_bar.selectedIndex){
                    var noted = note_editor.createObject(root);
                    noted.openNote(dataModel.data(indexPath));
                }
            }
            listItemComponents: [
                ListItemComponent {
                    CustomListItem {
                        id: item
                        dividerVisible: true
                        highlightAppearance: HighlightAppearance.Full
                        
                        Container {
                            topPadding: item.ListItem.view.to.sdu(1)
                            rightPadding: item.ListItem.view.to.sdu(1)
                            leftPadding: item.ListItem.view.to.sdu(1)
                            Label {
                                text: ListItemData.Title
                                textStyle.fontSize: FontSize.XLarge
                            }
                            contextActions: [
                                ActionSet {
                                    actions: [
                                        DeleteActionItem {
                                            onTriggered: {
                                                if(!item.ListItem.view.getSelectedIndex()){
                                                    item.ListItem.view.deleteNote(item.ListItem.view.selected());
                                                }
                                                else{
                                                    item.ListItem.view.deleteCategory(item.ListItem.view.selected());
                                                }
                                            }
                                        
                                        }
                                    ]
                                }
                            ]
                            horizontalAlignment: HorizontalAlignment.Fill
                            verticalAlignment: VerticalAlignment.Fill
                        }   
                    }
                }
            ]
        }            
    }
}
